(function (Drupal, once, drupalSettings) {
  'use strict';

  function toggleDisplay(element) {
    if (element.style.display === 'none') {
      element.style.display = '';
    } else {
      element.style.display = 'none';
    }
  }

  function buildHeadingMarkers(headingElements, highlightDefault) {
    return headingElements.map((heading) => {
      const tagName = heading.tagName.toLowerCase();
      const marker = document.createElement('span');
      marker.classList = [
        'heading-highlighter__heading',
        'heading-highlighter__heading--' + tagName,
      ].join(' ');
      marker.textContent = tagName.toUpperCase();
      if (!highlightDefault) {
        marker.style.display = 'none';
      }
      heading.prepend(marker);
      return marker;
    });
  }

  function buildToggle(config, markers) {
    if (!config['show_toggle']) {
      return;
    }
    once('heading-highlighter-toggle-init', 'body').forEach((body) => {
      // Build toggle button.
      const button = document.createElement('button');
      button.setAttribute('id', config['id']);
      button.classList.add('heading-highlighter-toggle');
      button.classList.add(config['toggle_placement']);
      const toggleStyles = [];
      button.setAttribute('style', toggleStyles.join('; '));
      button.append(config['toggle_text']);

      // Append toggle button to page.
      body.append(button);

      // Add toggle action.
      button.addEventListener('click', (e) => {
        markers.forEach((marker) => toggleDisplay(marker));
      });
    });
  }

  Drupal.behaviors.heading_highlighter = {
    attach: function (context, settings) {
      if (!settings.headingHighlighter) return;
      const config = settings.headingHighlighter;

      // Check for heading container.
      const selector = config['selector'] || 'body';
      const containers = once('heading-highlighter-init', selector, context);
      if (!containers.length) {
        console.warn(
          `Heading highlighter could not find container element matching: ${selector}`
        );
        return;
      }

      const headingQuery = 'h1, h2, h3, h4, h5, h6';
      let markers = [];

      // Process containers.
      containers.forEach((container) => {
        // Query for heading elements inside container.
        const headingElements = [...container.querySelectorAll(headingQuery)];
        if (!headingElements.length) return;

        // Generate heading markers and append to list.
        markers = markers.concat(
          buildHeadingMarkers(headingElements, config['highlight_default'])
        );
      });

      if (!markers.length) {
        console.warn(
          `Heading highlighter could not find heading elements inside: ${selector}`
        );
        return;
      }

      buildToggle(config, markers);
    },
    detach: function (context, settings) {},
  };
})(Drupal, once, drupalSettings);
